/* client2.c */

#include <sql.h>
#include <sqlext.h>
#include <sqltypes.h>
#include <stdio.h>

typedef enum { FALSE, TRUE } bool;

static bool SQL_OK( SQLRETURN result )
{
  if( result == SQL_SUCCESS || result == SQL_SUCCESS_WITH_INFO )
    return( TRUE );
  else
    return( FALSE );
}

static bool printErrors( SQLHENV  envHandle,
                         SQLHDBC  conHandle,
                         SQLHSTMT stmtHandle )
{
  SQLRETURN   result;
  SQLCHAR     sqlState[6];
  SQLINTEGER  nativeError;
  SQLSMALLINT requiredLength;
  SQLCHAR     messageText[SQL_MAX_MESSAGE_LENGTH+1];

  do
  {
    result = SQLError( envHandle,
                       conHandle,
                       stmtHandle,
                       sqlState,
                       &nativeError,
                       messageText,
                       sizeof( messageText ),
                       &requiredLength );

    if( SQL_OK( result ))
      {
        printf( "SQLState     = %s\n", sqlState );
        printf( "Native error = %d\n", nativeError );
        printf( "Message text = %s\n", messageText );
      }
  } while( SQL_OK( result ));
}

int main( int argc, char * argv[] )
{
  SQLRETURN   res;
  SQLHENV     env;
  SQLHDBC     con;
  SQLCHAR     fullConnectStr[SQL_MAX_OPTION_STRING_LENGTH];
  SQLSMALLINT requiredLength;

  res = SQLAllocHandle( SQL_HANDLE_ENV, SQL_NULL_HANDLE, &env );

  if( SQL_OK( res ))
  {
    res = SQLSetEnvAttr( env, 
                         SQL_ATTR_ODBC_VERSION, 
                         (SQLPOINTER)SQL_OV_ODBC2, 
                         0 );  
    if( !SQL_OK( res ))
    {
      printErrors( env, SQL_NULL_HDBC, SQL_NULL_HSTMT );
      exit( -1 );
    }

    res = SQLAllocHandle( SQL_HANDLE_DBC, env, &con );
    if( !SQL_OK( res ))
    {
      printErrors( env, SQL_NULL_HDBC, SQL_NULL_HSTMT );
      exit( -2 );
    }

    res = SQLDriverConnect( con, 
                            (SQLHWND)NULL,
                            argv[1], SQL_NTS,
                            fullConnectStr, 
                            sizeof( fullConnectStr ),
                            &requiredLength,
                            SQL_DRIVER_NOPROMPT );

                              
    if( !SQL_OK( res ))
    {
      printErrors( SQL_NULL_HENV, con, SQL_NULL_HSTMT );
      exit( -3 );
    }

    printf( "connection ok\n" );
    printf( "full connection string = '%s'\n", fullConnectStr );
    printf( "disconnecting\n" );

    res = SQLDisconnect( con );
    if( !SQL_OK( res ))
    {
      printErrors( SQL_NULL_HENV, con, SQL_NULL_HSTMT );
      exit( -4 );
    }

    res = SQLFreeHandle( SQL_HANDLE_DBC, con );
    if( !SQL_OK( res ))
    {
      printErrors( SQL_NULL_HENV, con, SQL_NULL_HSTMT );
      exit( -5 );
    }

    res = SQLFreeHandle( SQL_HANDLE_ENV, env );
    if( !SQL_OK( res ))
    {
      printErrors( env, SQL_NULL_HDBC, SQL_NULL_HSTMT );
      exit( -6 );
    }
  }

  exit( 0 );

}
