/*
** File: client2b.c
*/

#include <stdlib.h>
#include <libpq-fe.h>

static void show_connection_attributes( const PGconn * conn );
static const char * check( const char * value );

int main( int argc, char * argv[] )
{
  PGconn * connection;

  if( argc != 2 )
  {
    printf( "usage  : %s \"connection-string\"\n", argv[0] );
    printf( "example: %s \"user=myname password=cows\"\n", argv[0] );
    exit( 1 );
  }

  if(( connection = PQconnectdb( argv[1] )) == NULL )
  {
    printf( "Fatal error - unable to allocate connection\n" );
    exit( 1 );
  }	

  if( PQstatus( connection ) != CONNECTION_OK )
    printf( "%s\n", PQerrorMessage( connection ));
  else
    printf( "Connection ok\n" );

  show_connection_attributes( connection );

  PQfinish( connection );

  exit( 0 );

}

static const char * check( const char * value )
{
    if( value )
	return( value );
    else
	return( "(null)" );
}

static void show_connection_attributes( const PGconn * c )
{
  printf( "dbname   = %s\n", check( PQdb( c )));
  printf( "user     = %s\n", check( PQuser( c )));
  printf( "password = %s\n", check( PQpass( c )));
  printf( "host     = %s\n", check( PQhost( c )));	
  printf( "port     = %s\n", check( PQport( c )));
  printf( "tty      = %s\n", check( PQtty( c )));
  printf( "options  = %s\n", check( PQoptions( c )));
}
