/*
** File: client3.c
*/

#include <stdlib.h>
#include <libpq-fe.h>

void process_query( PGconn * connection, const char * query_text )
{
  PGresult	*	result;
  PQprintOpt		options = {0};

  if(( result = PQexec( connection, query_text )) == NULL )
  {
    printf( "%s\n", PQerrorMessage( connection ));
    return;
  }
    
  options.header    = 1;    /* Ask for column headers		   */
  options.align     = 1;    /* Pad short columns for alignment   */
  options.fieldSep  = "|";  /* Use a pipe as the field separator */

  PQprint( stdout, result, &options );
  
  PQclear( result );
}

int main( int argc, char * argv[] )
{
  PGconn * connection;

  if( argc != 2 )
  {
    printf( "usage  : %s \"connection-string\"\n", argv[0] );
    printf( "example: %s \"user=myname password=cows\"\n", argv[0] );
    exit( 1 );
  }

  if(( connection = PQconnectdb( argv[1] )) == NULL )
  {
    printf( "Fatal error - unable to allocate connection\n" );
    exit( 1 );
  }

  if( PQstatus( connection ) != CONNECTION_OK )
    printf( "%s\n", PQerrorMessage( connection ));
  else
  {
    process_query( connection, "SELECT * FROM rentals" );
  }	

  PQfinish( connection );

  exit( 0 );
}
