#!/usr/local/bin/wish
#
# Filename: client3.tcl

proc main { } {

  wm withdraw .

  package require Tktable

  set conn [connect]

  if { $conn != {} } {

    build_dialog $conn
             
    tkwait window .top 
 
    pg_disconnect $conn
  }
}

proc build_dialog { conn } {

  toplevel .top 

  wm title .top "client3"

  set table [make_table .top]

  button .top.close -text "Close Window" -command {exit}

  label .top.label   -text "Enter an SQL Command and Press Return"
  text  .top.command -height 3
  label .top.status 

  focus -force .top.command

  bind .top.command <Return> \
      "process_command $conn $table \[.top.command get 1.0 end\]"

  scrollbar .top.sy -command [list $table yview]
  scrollbar .top.sx -command [list $table xview] -orient horizontal

  grid     .top.label                     
  grid    .top.command      -sticky news  
  grid     .top.status                    
  grid   $table .top.sy     -sticky news  
  grid      .top.sx         -sticky ew    
  grid     .top.close                     
  
  grid columnconfig .top 0 -weight 1    

  grid rowconfig    .top 1 -weight 0    
  grid rowconfig    .top 3 -weight 1    
  grid rowconfig    .top 5 -weight 0    
}

proc make_table { parent } { 

  table $parent.table \
    -variable table_data \
    -yscrollcommand {.top.sy set} \
    -xscrollcommand {.top.sx set} \
    -titlerows 1 \
    -titlecols 1 \
    -roworigin -1 \
    -colorigin -1 \
    -colstretchmode last -rowstretchmode last

  return $parent.table

}

proc process_command { conn table command } { 

  set result_set [pg_exec $conn $command]

  switch [pg_result $result_set -status] {

    PGRES_EMPTY_QUERY {
      .top.status configure -text ""
    }

    "PGRES_TUPLES_OK" { 
      .top.status configure -text "Ok"
      load_table $table $result_set 
    }

    "PGRES_COMMAND_OK" {    
      .top.status configure -text "Ok"
    }

    default
    {
      .top.status configure -text ""

      tk_messageBox -title [pg_result $result_set -status] \
                    -message [pg_result $result_set -error] \
                    -type ok
    }
  }
}

proc load_table { table result_set } {

  size_table $table $result_set

  set_column_headers $table $result_set

  fill_table $table $result_set
    
  size_columns $table $result_set
}

proc size_table { table result_set } {

  set col_cnt   [pg_result $result_set -numAttrs]
  set row_cnt   [pg_result $result_set -numTuples]

  $table configure \
    -rows [expr $row_cnt + 1 ] \
    -cols [expr $col_cnt + 1 ] 
}

proc set_column_headers { table result_set } {
  
  global col_widths

  set col_cnt   [pg_result $result_set -numAttrs]
  set col_names [pg_result $result_set -attributes]

  for {set col 0} {$col < $col_cnt} {incr col} {
    set col_name [lindex $col_names $col]
    $table set -1,$col $col_name
    set col_widths($col) [string length $col_name]
  }
}

proc fill_table { table result_set } {

  global col_widths

  set col_cnt   [pg_result $result_set -numAttrs]
  set row_cnt   [pg_result $result_set -numTuples]

  for {set row 0} {$row < $row_cnt} {incr row} {
    set tuple [pg_result $result_set -getTuple $row]

    $table set $row,-1 [expr $row + 1]

    for {set col 0} {$col < $col_cnt} {incr col} {
    
      set val [lindex $tuple $col]

      if { $col_widths($col) < [string length $val] } {
        set col_widths($col) [string length $val]
      }
      $table set $row,$col $val
    }
  }
}

proc size_columns { table result_set } {

  global col_widths

  set col_cnt   [pg_result $result_set -numAttrs]

  for {set col 0} {$col < $col_cnt} {incr col} {
    $table width $col $col_widths($col)
  }

  $table width -1 5

}

#  Mainline code follows
#
source pgconnect.tcl
main
exit
