/*
** File: client3b.c
*/

#include <stdlib.h>
#include <libpq-fe.h>

void process_query( PGconn * connection, const char * query_text )
{
  PGresult	*	result;

  if(( result = PQexec( connection, query_text )) == NULL )
  {
    printf( "%s\n", PQerrorMessage( connection ));
    return;
  }

  if(( PQresultStatus( result ) == PGRES_COMMAND_OK ) ||
     ( PQresultStatus( result ) == PGRES_TUPLES_OK ))
  {    
    PQprintOpt		options = {0};

    options.header    = 1;    /* Ask for column headers		   */
    options.align     = 1;    /* Pad short columns for alignment   */
    options.fieldSep  = "|";  /* Use a pipe as the field separator */

    PQprint( stdout, result, &options );

  }
  else
  {
    printf( "%s\n", PQresStatus( PQresultStatus( result )));
    printf( "%s\n", PQresultErrorMessage( result ));

  }

  PQclear( result );
}

int main( int argc, char * argv[] )
{
  PGconn * connection;

  if( argc != 2 )
  {
    printf( "usage  : %s \"connection-string\"\n", argv[0] );
    printf( "example: %s \"user=myname password=cows\"\n", argv[0] );
    exit( 1 );
  }

  if(( connection = PQconnectdb( argv[1] )) == NULL )
  {
    printf( "Fatal error - unable to allocate connection\n" );
    exit( 1 );
  }

  if( PQstatus( connection ) != CONNECTION_OK )
    printf( "%s\n", PQerrorMessage( connection ));
  else
  {
    process_query( connection, "SELECT * FROM rentals" );
  }	

  PQfinish( connection );

  exit( 0 );
}
