/*
** File: client3c.c
*/

#include <stdlib.h>
#include <libpq-fe.h>

void print_meta_data( PGresult * result )
{
  int	col;

  printf( "Status: %s\n", PQresStatus( PQresultStatus( result )));
  printf( "Returned %d rows ", PQntuples( result ));
  printf( "with %d columns\n\n", PQnfields( result ));

  printf( "Column Type TypeMod Size Name       \n" );
  printf( "------ ---- ------- ---- -----------\n" );

  for( col = 0; col < PQnfields( result ); col++ )
  {
    printf( "%3d    %4d %7d %4d %s\n",
	    col,
	    PQftype( result, col ),
	    PQfmod( result, col ),
	    PQfsize( result, col ),
	    PQfname( result, col ));
  }
}

void process_query( PGconn * connection, const char * query_text )
{
  PGresult	*	result;

  if(( result = PQexec( connection, query_text )) == NULL )
  {
    printf( "%s\n", PQerrorMessage( connection ));
    return;
  }

  if(( PQresultStatus( result ) == PGRES_COMMAND_OK ) ||
     ( PQresultStatus( result ) == PGRES_TUPLES_OK ))
  {    
    PQprintOpt		options = {0};

    options.header    = 1;    /* Ask for column headers		   */
    options.align     = 1;    /* Pad short columns for alignment   */
    options.fieldSep  = "|";  /* Use a pipe as the field separator */

    PQprint( stdout, result, &options );

  }
  
  print_meta_data( result );

  PQclear( result );
}

int main( int argc, char * argv[] )
{
  PGconn * connection;

  if( argc != 2 )
  {
    printf( "usage  : %s \"connection-string\"\n", argv[0] );
    printf( "example: %s \"user=myname password=cows\"\n", argv[0] );
    exit( 1 );
  }

  if(( connection = PQconnectdb( argv[1] )) == NULL )
  {
    printf( "Fatal error - unable to allocate connection\n" );
    exit( 1 );
  }

  if( PQstatus( connection ) != CONNECTION_OK )
    printf( "%s\n", PQerrorMessage( connection ));
  else
  {
    process_query( connection, "SELECT * FROM rentals" );
  }	

  PQfinish( connection );

  exit( 0 );
}
