/*
** File: client3d.c
*/

#include <stdlib.h>
#include <string.h>
#include <libpq-fe.h>

#define MAX_PRINT_LEN	40

static char separator[MAX_PRINT_LEN+1];

void print_result_set( PGresult * result )
{
  int          col;
  int          row;
  int	     * sizes;

/*
**  Compute the size for each column
*/
  sizes = (int *)calloc( PQnfields( result ), sizeof( int ));
  
  for( col = 0; col < PQnfields( result ); col++ )
  {
    int	len = 0;

    for( row = 0; row < PQntuples( result ); row++ )
    {
      if( PQgetisnull( result, row, col ))
        len = 0;
      else
	len = PQgetlength( result, row, col );

      if( len > sizes[col] )
	sizes[col] = len;
    }

    if(( len = strlen( PQfname( result, col ))) > sizes[col] )
      sizes[col] = len;

    if( sizes[col] > MAX_PRINT_LEN )
      sizes[col] = MAX_PRINT_LEN;
  }

/*
**  Print the field names.
*/
  for( col = 0; col < PQnfields( result ); col++ )
  {
    printf( "%-*s ", sizes[col], PQfname( result, col ));
  }

  printf( "\n" );

/*
**  Print the separator line
*/
  memset( separator, '-', MAX_PRINT_LEN );

  for( col = 0; col < PQnfields( result ); col++ )
  {   
    printf( "%*.*s ", sizes[col], sizes[col], separator );
  }

  printf( "\n" );

/*
**  Now loop through each of the tuples returned by
**  our query and print the results.
*/
  for( row = 0; row < PQntuples( result ); row++ )
  {
    for( col = 0; col < PQnfields( result ); col++ )
    {
      if( PQgetisnull( result, row, col ))
	printf( "%*s", sizes[col], "" );
      else
	printf( "%*s ", sizes[col], PQgetvalue( result, row, col ));
    }

    printf( "\n" );

  }
  printf( "(%d rows)\n", PQntuples( result ));
  free( sizes );
}

void process_query( PGconn * connection, const char * query_text )
{
  PGresult	*	result;

  if(( result = PQexec( connection, query_text )) == NULL )
  {
    printf( "%s\n", PQerrorMessage( connection ));
    return;
  }

  if( PQresultStatus( result ) == PGRES_TUPLES_OK )
  {
    print_result_set( result );
  }
  else if( PQresultStatus( result ) == PGRES_COMMAND_OK )
  {    
    printf( "%s", PQcmdStatus( result ));

    if( strlen( PQcmdTuples( result )))
      printf( " - %s rows\n", PQcmdTuples( result ));
    else
      printf( "\n" );
  }
  else
  {
      printf( "%s\n", PQresultErrorMessage( result ));
  }

  PQclear( result );
}

int main( int argc, char * argv[] )
{
  PGconn * connection;

  if( argc != 2 )
  {
    printf( "usage  : %s \"connection-string\"\n", argv[0] );
    printf( "example: %s \"user=myname password=cows\"\n", argv[0]);
    exit( 1 );
  }

  if(( connection = PQconnectdb( argv[1] )) == NULL )
  {
    printf( "Fatal error - unable to allocate connection\n" );
    exit( 1 );
  }

  if( PQstatus( connection ) != CONNECTION_OK )
    printf( "%s\n", PQerrorMessage( connection ));
  else
    process_query( connection, "SELECT * FROM rentals" );

  PQfinish( connection );

  exit( 0 );
}
