#!/usr/bin/perl -W
#
#  Filename: client3e.pl
#
use strict;
use DBI;

my $dbh = DBI->connect("dbi:Pg:", undef, undef, {PrintError => 1})
  or die "Can't connect to PostgreSQL: $DBI::errstr ($DBI::err)\n";

my $sth = $dbh->prepare( $ARGV[0] );

if( defined( $sth )) {
  if( $sth->execute()) {
    my($widths, $row_values) = compute_column_widths( $sth );
    print_column_headings( $sth, $widths );
    print_results( $row_values, $widths );
  }
}

$dbh->disconnect();

#
#  subroutine: compute_column_widths( $sth )
#
sub compute_column_widths
{
  my $sth   = shift;
  my $names = $sth->{NAME};
  my @widths;

  for( my $col = 0; $col < $sth->{NUM_OF_FIELDS}; $col++ ) {
    push( @widths, length( $names->[$col] ));
  }

  my $row_values = $sth->fetchall_arrayref();

  for( my $col = 0; $col < $sth->{NUM_OF_FIELDS}; $col++ ) {
    for( my $row = 0; $row < $sth->rows(); $row++ ) {
      if( defined( $row_values->[$row][$col] )) {
        if( length( $row_values->[$row][$col] ) > $widths[$col] ) {
	  $widths[$col] = length( $row_values->[$row][$col] );
        }
      }
    }
  }

  return( \@widths, $row_values );
}

#
#  subroutine: pad( $val, $col_width, $pad_char )
#
sub pad
{
  my( $val, $col_width, $pad_char ) = @_;
  my $pad_len;

  $val      = "" if ( !defined( $val ));
  $pad_char = " " if( !defined( $pad_char ));
  $pad_len  = $col_width - length( $val );

  return( $val . $pad_char x $pad_len . " " );

}

#
#  subroutine: print_column_headings( $sth )
#
sub print_column_headings
{
  my $sth    = shift;
  my $widths = shift;
  my $names  = $sth->{NAME};
    
  for( my $col = 0; $col < $sth->{NUM_OF_FIELDS}; $col++ ) {
    print( pad( $names->[$col], $widths->[$col] ));
  }

  print( "\n" );

  for( my $col = 0; $col < $sth->{NUM_OF_FIELDS}; $col++ ) {
    print( pad( "-", $widths->[$col], "-" ));
  }

  print( "\n" );
}

#
#  subroutine: print_results( )
#
sub print_results
{
  my( $rows, $widths ) = @_;

  for( my $row = 0; $row < $sth->rows(); $row++ ) {
    for( my $col = 0; $col < $sth->{NUM_OF_FIELDS}; $col++ ) {
      print( pad( $rows->[$row][$col], $widths->[$col] ));
    }
    print( "\n" );
  }
}
