//
//  File: client4.java
//

import java.sql.*;
import java.io.*;

public class client4
{
  static String blanks = "                                   ";
  static String dashes = "-----------------------------------";

  public static void main( String args[] )
    throws SQLException
  {
    Class driverClass = loadDriver( "org.postgresql.Driver" );

    if( driverClass == null )
      return;

    if( args.length != 1 )
    {
      System.err.println( "usage: java client4 <url>" );
      return;
    }

    Connection con = connectURL( args[0] );

    if( con != null )
    {
      DatabaseMetaData dbmd = con.getMetaData();

      System.out.print( "Connected to " );
      System.out.print( dbmd.getDatabaseProductName());
      System.out.println( " " + dbmd.getDatabaseProductVersion());

      processCommands( con );

      con.close();
    }
  }

  static void processCommands( Connection con )
  {    
    try
    {
      Statement      stmt = con.createStatement();
      String         cmd  = "";
      BufferedReader in;

      in = new BufferedReader( new InputStreamReader( System.in ));

      while( true )
      {
        System.out.print( "--> ");
	
	cmd = in.readLine();

	if( cmd == null )
	    break;

	if( cmd.equalsIgnoreCase( "quit" ))
	  break;

        processCommand( stmt, cmd );

      }
      
      System.out.println( "bye" );

    }
    catch( Exception e )
    {
	System.err.println( e );
    }
  }

  static void processCommand( Statement stmt, String cmd )
    throws SQLException
  {

    if( stmt.execute( cmd ))
	printResultSet( stmt.getResultSet());
    else 
    {
      int count = stmt.getUpdateCount();    

      if( count == -1 )
	System.out.println( "No results returned" );
      else
	System.out.println( "(" + count + " rows)" );
    }
  }

  static String pad( String in, int len, String fill )
  {
      String result = in;

      len -= in.length();

      while( len > 0  )
      {
	  int l;

	  if( len > fill.length())
	      l = fill.length();
	  else
	      l = len;

	result = result + fill.substring( 0, l );

	len -= l;
      }

      return( result );
  }

  static void printResultSet( ResultSet rs )
    throws SQLException
  {
    int[] 	      sizes;
    ResultSetMetaData rsmd     = rs.getMetaData();
    int		      colCount = rsmd.getColumnCount();
    int		      rowCount = 0;
    
    sizes = new int[colCount+1];

    //
    // Compute column widths
    //
    while( rs.next())
    {
      rowCount++;

      for( int i = 1; i <= colCount; i++ )
      {
	String val = rs.getString(i);
		  
	if(( rs.wasNull() == false ) && ( val.length() > sizes[i] ))
	  sizes[i] = val.length();
      }
    }

    //
    // Print column headers
    //
    for( int i = 1; i <= colCount; i++ )
    {
      if( rsmd.getColumnLabel(i).length() > sizes[i] )
	sizes[i] = rsmd.getColumnLabel(i).length();

      System.out.print( pad( rsmd.getColumnLabel( i ), 
			     sizes[i], 
			     blanks ));

      if( i < colCount )
	System.out.print( " | " );
      else
	System.out.println();
    }	

    for( int i = 1; i <= colCount; i++ )
    {
      if( i < colCount )
	System.out.print( pad( "", sizes[i], dashes ) + "-+-" );
      else
	System.out.println( pad( "", sizes[i], dashes ));		  
    }	

    //
    //  Rewind the result set and print the contents
    //
    rs.beforeFirst();
      
    while( rs.next())
    {
      for( int i = 1; i <= colCount; i++ )
      {
        String val = rs.getString(i);
		  
	if( rs.wasNull())
	  val = "";

	if( i < colCount )
	  System.out.print( pad( val, sizes[i], blanks ) + " | " );
	else
	  System.out.println( pad( val, sizes[i], blanks ));
      }
    }
  }

  static Class loadDriver( String driverName )
  {
    try 
    { 
      return( Class.forName( driverName ));
    }
    catch( ClassNotFoundException e )
    {
      System.err.println( "Can't load driver - " + e.getMessage());
      return( null );
    }
  }

  static Connection connectURL( String URL )
  {
    try
    {
      return( DriverManager.getConnection( URL ));
    }
    catch( SQLException e )
    {
      System.err.println( "Can't connect - " + e.getMessage());
      return( null );
    }
  }
}
