#!/usr/bin/python
#
# File: client4.py

import pgdb
from Tkinter import *
from Tktable import Table,ArrayVar

class PGDialog:
#############################################
    def __init__( self ):
        self.widths  = []
        self.conn    = None
        # Widgets
        self.table   = None
        self.command = None
        self.status  = None

#############################################
    def main( self ):

        self.conn = pgdb.connect( database="movies" )
    
        if( self.conn != None ):
            self.build_dialog( )
            self.table.mainloop()

#############################################
    def build_dialog( self ):

        root = Tk()

        self.make_table( root )
        
        self.command = Text( root, height=3 )
        self.status  = Label( root )
    
        close = Button( root,
                        text="Close Window",
                        command=root.destroy )
        
        label = Label( root,
                       text="Enter an SQL Command and Press Return")

        self.command.focus_force( )
    
        self.command.bind( "<Return>", self.execute )
    
        sy = Scrollbar( root,
                        command=self.table.yview )
        
        sx = Scrollbar( root,
                        command=self.table.xview,
                        orient="horizontal" )

        self.table.config( xscrollcommand=sx.set,
                           yscrollcommand=sy.set )
    
        label.grid( row=0 )

        self.command.grid( row=1, sticky='news' )
        self.status.grid( row=2 )
        self.table.grid( row=3, column=0, sticky='news' )

        sy.grid( row=3, column=1, sticky='news' )
        sx.grid( row=4, sticky='ew' )
        close.grid( row=5 )

        root.columnconfigure( 0, weight=1 )

        root.rowconfigure( 1, weight=0 )
        root.rowconfigure( 3, weight=1 )
        root.rowconfigure( 5, weight=0 )

#############################################
    def make_table( self, parent ):

        var = ArrayVar( parent )

        self.table = Table( parent,
                            variable=var,
                            titlerows=1,
                            titlecols=1,
                            roworigin=-1,
                            colorigin=-1,
                            colstretchmode='last',
                            rowstretchmode='last' )

#############################################
    def execute( self, event ):

        self.process_command( self.command.get( "1.0", "end" ))

#############################################
    def process_command ( self, command ):

        cur = self.conn.cursor()
        
        try:
            cur.execute( command )
            self.load_table( cur )
            
        except Exception, e:
            from mx.TextTools import collapse
            self.status.configure( text=collapse( str( e )))


#############################################
    def load_table( self, cur ):

        self.status.configure( text="%d row(s)" % cur.rowcount )
        self.size_table( cur )

        if( cur.description == None ):
            return
        
        self.set_column_headers( cur )

        self.fill_table( cur )

        self.size_columns( cur )

#############################################    
    def size_table( self, cur ):

        if( cur.description == None ):
            self.table.configure( rows=0, cols=0 )
        else:
            col_cnt = len( cur.description )
            row_cnt = cur.rowcount

            self.table.configure( rows=row_cnt+1, cols=col_cnt+1 )
    
#############################################
    def set_column_headers( self, cur ):

        col_no = 0

        for col in cur.description:
            self.table.set( "-1," + str(col_no), col[0] )
            self.widths.append(len( col[0] ))
            col_no = col_no + 1


#############################################
    def fill_table( self, cur ):

        rows = cur.fetchall()

        r = 0
        for row in rows:
            c = 0

            for col in row:

                self.table.set( str(r) + "," + str(c), str( col ))

                if( col != None ):
                    if( len( str( col )) > self.widths[c] ):
                        self.widths[c] = len( str( col ))

                c = c + 1

            self.table.set( str(r) + ",-1", str(r))
            
            r = r + 1
    
#############################################
    def size_columns( self, cur ):
        col_cnt = len( cur.description )

        for col in range( 0, col_cnt ):
            self.table.width( col, self.widths[col] )

#############################################

obj = PGDialog()
obj.main( )
