/*
**  Filename:  filelist.c
*/

#include "postgres.h"
#include "fmgr.h"
#include "nodes/execnodes.h"

#include <dirent.h>

typedef struct
{
  int              dir_ctx_count;
  int              dir_ctx_current;
  struct dirent ** dir_ctx_entries;
} dir_ctx;

PG_FUNCTION_INFO_V1(filelist);

Datum filelist(PG_FUNCTION_ARGS)
{
  FmgrInfo      * fmgr_info  = fcinfo->flinfo;
  ReturnSetInfo * resultInfo = (ReturnSetInfo *)fcinfo->resultinfo;
  text          * startText  = PG_GETARG_TEXT_P(0);
  int             len        = VARSIZE( startText ) - VARHDRSZ;
  char          * start      = (char *)palloc( len+1 );
  dir_ctx       * ctx;
  
  memcpy( start, startText->vl_dat, len );
  start[len] = '\0';
  
  if( fcinfo->resultinfo == NULL ) 
    elog(ERROR, "filelist: context does not accept a set result");

  if( !IsA( fcinfo->resultinfo, ReturnSetInfo ))
    elog(ERROR, "filelist: context does not accept a set result");
  
  if( fmgr_info->fn_extra == NULL )
  {
    dir_ctx     * new_ctx;
    
    fmgr_info->fn_extra = MemoryContextAlloc( fmgr_info->fn_mcxt, 
                                              sizeof( dir_ctx ));
    
    new_ctx = (dir_ctx *)fmgr_info->fn_extra;
    
    new_ctx->dir_ctx_count   = scandir( start, 
                                        &new_ctx->dir_ctx_entries, 
                                        NULL, 
                                        alphasort );
    new_ctx->dir_ctx_current = 0;
  }
  
  ctx = (dir_ctx *)fmgr_info->fn_extra;
  
  if( ctx->dir_ctx_count == -1 )
  {
    pfree( fmgr_info->fn_extra );
    
    fmgr_info->fn_extra = NULL;
    
    resultInfo->isDone = ExprEndResult;
    
    PG_RETURN_NULL();
  }
  
  if( ctx->dir_ctx_current < ctx->dir_ctx_count )
  {
    struct dirent * entry;
    size_t          nameLen;
    size_t          resultLen;
    text          * result;

    entry     = ctx->dir_ctx_entries[ctx->dir_ctx_current];
    nameLen   = strlen( entry->d_name );
    resultLen = nameLen + VARHDRSZ;

    result = (text *)palloc( resultLen );
    
    VARATT_SIZEP( result ) = resultLen;
    
    memcpy( VARDATA( result ), entry->d_name, nameLen );
    
    resultInfo->isDone = ExprMultipleResult;

/*
**  Advance to the next entry in our array of
**  filenames/subdirectories
*/
    ctx->dir_ctx_current++;
  
    PG_RETURN_TEXT_P( result );
  }
  else
  {
    free( ctx->dir_ctx_entries );
    
    pfree( fmgr_info->fn_extra );
    
    fmgr_info->fn_extra = NULL;
    
    resultInfo->isDone = ExprEndResult;
    
    PG_RETURN_NULL();
  }
}
