/* qt-cursor.cpp */

#include <qapplication.h>    // QT Basic application classes
#include <qwidget.h>         // QT Basic widget class
#include <qtable.h>          // QT Table widget
#include <qmessagebox.h>     // QT MessageBox widget
#include <qlayout.h>         // QT Layout manager
#include <qpushbutton.h>     // QT Pushbutton widget
#include <qhbuttongroup.h>   // QT Button group widget

#include <libpq++.h>         // PostgreSQL libpq++ API 
#include <iostream.h>        // Standard C++ io library

#include "qt-cursor-test.moc"

void main( int argc, char * argv[] )
{
  QApplication app( argc, argv );
  MyMain       win( app.argv()[2], app.argv()[1] );
  
  app.setMainWidget( &win );
  
  win.show();
  app.exec();
}

MyMain::MyMain( const char * connect, const char * query )
{
  QVBoxLayout   * vbox  = new QVBoxLayout( this );
  QHButtonGroup * group = new QHButtonGroup( this );  

  table = new MyTable( this, connect ? connect : "", query ? query : "" );
  
  new QPushButton( "Quit", group );       // id = 0
  new QPushButton( "Fetch All", group );  // id = 1
  new QPushButton( "Fetch Prev", group ); // id = 2
  new QPushButton( "Fetch Next", group ); // id = 3
  new QPushButton( "Prev 5", group );     // id = 4
  new QPushButton( "Next 5", group );     // id = 5

  vbox->addWidget( table );
  vbox->addWidget( group );

  connect( group, SIGNAL( clicked( int )), 
	   table, SLOT( fetch( int )));
}

void MyTable::fetch( int id )
{
  int result;

  switch( id )
  {
    case 0:
      QApplication::exit( 0 );
      break;

    case 1:  // Fetch All
      result = cursor->Fetch();
      break;

    case 2:  // Fetch Previous
      result = cursor->Fetch( 1, "backward" );
      break;

    case 3:  // Fetch Next
      result = cursor->Fetch( 1, "forward" );
      break;

    case 4:  // Fetch Previous 5
      result = cursor->Fetch( 5, "backward" );
      break;

    case 5:  // Fetch Next 5
      result = cursor->Fetch( 5, "forward" );
      break;
  }

  if( result == 0 )
  {
    QMessageBox::critical( 0, "fetch failed", 
			   cursor->ErrorMessage());
  }
  else
  {
    buildTable();
  }
}

void MyTable::buildTable( void )
{
  setNumRows( cursor->Tuples());
  setNumCols( cursor->Fields());

  for( int col = 0; col < cursor->Fields(); col++ )
  {
      horizontalHeader()->setLabel( col, cursor->FieldName( col ));
  }

  for( int row = 0; row < cursor->Tuples(); row++ )
  {
    for( int col = 0; col < cursor->Fields(); col++ )
    {
      setText( row, col, cursor->GetValue( row, col ));
    }
  }
}

MyTable::MyTable( QWidget * parent, 
		  const char * connect, 
		  const char * query ) 
  : QTable( parent )
{
  cursor = new PgCursor( connect, "my_cursor" );

  if( cursor->ConnectionBad())
  {
    QMessageBox::critical( 0, "Connection failed", 
			   cursor->ErrorMessage());
    exit( -1 );
  }

  if( !cursor->Declare( query ))
  {
    QMessageBox::critical( 0, "Query failed", 
			   cursor->ErrorMessage());
    exit( -1 );
  }
}
