/* qt-query.cpp */

#include <qapplication.h>    // QT Basic application classes
#include <qwidget.h>         // QT Basic widget class
#include <qtable.h>          // QT Table widget
#include <qmessagebox.h>     // QT MessageBox widget
#include <qlayout.h>         // QT Layout manager
#include <qpushbutton.h>     // QT Pushbutton widget

#include <libpq++.h>         // PostgreSQL libpq++ API 
#include <iostream.h>        // Standard C++ io library

#include "qt-query.h"

void main( int argc, char * argv[] )
{
  QApplication app( argc, argv );
  MyMain       win( app.argv()[2], app.argv()[1] );
  
  app.setMainWidget( &win );
  
  win.show();
  app.exec();
}

MyMain::MyMain( const char * argv1, const char * query )
{
  QVBoxLayout *  vbox    = new QVBoxLayout( this );

  table = new MyTable( this, argv1, query );
  
  QPushButton * quit = new QPushButton( "Quit", this );
  
  connect( quit, SIGNAL( clicked()), qApp, SLOT( quit()));

  vbox->addWidget( table );
  vbox->addWidget( quit );

}

MyTable::MyTable( QWidget * parent, 
		  const char * connect, 
		  const char * query ) 
  : QTable( parent )
{
  db = new PgDatabase( connect ? connect : "" );

  if( db->ConnectionBad())
  {
    QMessageBox::critical( 0, "Connection failed", 
			   db->ErrorMessage());
    exit( -1 );
  }

  if( db->ExecTuplesOk( query ? query : "" ))
  {
    setNumRows( db->Tuples());
    setNumCols( db->Fields());

    for( int col = 0; col < db->Fields(); col++ )
    {
	horizontalHeader()->setLabel( col, db->FieldName( col ));
    }

    for( int row = 0; row < db->Tuples(); row++ )
    {
      for( int col = 0; col < db->Fields(); col++ )
      {
	setText( row, col, db->GetValue( row, col ));
      }
    }
  }
  else
  {
    QMessageBox::critical( 0, "Query failed", db->ErrorMessage());
    exit( -1 );
  }
}





